<?php

namespace Suggestion\Suggestion\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Suggestion\Suggestion\Http\Requests\SuggestionsRequest;
use Suggestion\Suggestion\Interfaces\SuggestionsRepositoryInterface;
use Suggestion\Suggestion\Models\Suggestions;

/**
 * Resource controller class for suggestions.
 */
class SuggestionsResourceController extends BaseController
{

    /**
     * Initialize suggestions resource controller.
     *
     * @param type SuggestionsRepositoryInterface $suggestions
     *
     * @return null
     */
    public function __construct(SuggestionsRepositoryInterface $suggestions)
    {
        parent::__construct();
        $this->repository = $suggestions;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Suggestion\Suggestion\Repositories\Criteria\SuggestionsResourceCriteria::class);
    }

    /**
     * Display a list of suggestions.
     *
     * @return Response
     */
    public function index(SuggestionsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Suggestion\Suggestion\Repositories\Presenter\SuggestionsPresenter::class)
                ->$function();
        }

        $suggestions = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('suggestion::suggestions.names'))
            ->view('suggestion::suggestions.index', true)
            ->data(compact('suggestions', 'view'))
            ->output();
    }

    /**
     * Display suggestions.
     *
     * @param Request $request
     * @param Model   $suggestions
     *
     * @return Response
     */
    public function show(SuggestionsRequest $request, Suggestions $suggestions)
    {

        if ($suggestions->exists) {
            $view = 'suggestion::suggestions.show';
        } else {
            $view = 'suggestion::suggestions.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('suggestion::suggestions.name'))
            ->data(compact('suggestions'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new suggestions.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SuggestionsRequest $request)
    {

        $suggestions = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('suggestion::suggestions.name')) 
            ->view('suggestion::suggestions.create', true) 
            ->data(compact('suggestions'))
            ->output();
    }

    /**
     * Create new suggestions.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SuggestionsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $suggestions                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('suggestion::suggestions.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('suggestion/suggestions/' . $suggestions->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/suggestion/suggestions'))
                ->redirect();
        }

    }

    /**
     * Show suggestions for editing.
     *
     * @param Request $request
     * @param Model   $suggestions
     *
     * @return Response
     */
    public function edit(SuggestionsRequest $request, Suggestions $suggestions)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('suggestion::suggestions.name'))
            ->view('suggestion::suggestions.edit', true)
            ->data(compact('suggestions'))
            ->output();
    }

    /**
     * Update the suggestions.
     *
     * @param Request $request
     * @param Model   $suggestions
     *
     * @return Response
     */
    public function update(SuggestionsRequest $request, Suggestions $suggestions)
    {
        try {
            $attributes = $request->all();

            $suggestions->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('suggestion::suggestions.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('suggestion/suggestions/' . $suggestions->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('suggestion/suggestions/' . $suggestions->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the suggestions.
     *
     * @param Model   $suggestions
     *
     * @return Response
     */
    public function destroy(SuggestionsRequest $request, Suggestions $suggestions)
    {
        try {

            $suggestions->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('suggestion::suggestions.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('suggestion/suggestions/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('suggestion/suggestions/' . $suggestions->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple suggestions.
     *
     * @param Model   $suggestions
     *
     * @return Response
     */
    public function delete(SuggestionsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('suggestion::suggestions.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('suggestion/suggestions'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/suggestion/suggestions'))
                ->redirect();
        }

    }

    /**
     * Restore deleted suggestions.
     *
     * @param Model   $suggestions
     *
     * @return Response
     */
    public function restore(SuggestionsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('suggestion::suggestions.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/suggestion/suggestions'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/suggestion/suggestions/'))
                ->redirect();
        }

    }

}
